#!/usr/bin/python

# -------- more details -------- #

# http://oldwww.fysik.dtu.dk/campos/Documentation/Dacapo/dacapoNETCDF/dacapoNETCDF.html #
# http://oldwww.fysik.dtu.dk/CAMPOS/Documentation/Tutorials/dacapo_tutorial.html #
# http://oldwww.fysik.dtu.dk/campos/FAQ/FAQ.html #

# -------- packages -------- #

from Simulations.Dacapo import *
from Simulations.Dacapo.EigenState import EigenState
from Simulations.Dacapo.ListOfEigenStates import ListOfEigenStates

# -------- potentials -------- #

sim = Simulation()

Si.SetProperty("PseudopotentialPath",
		"/usr/local/dacapo/psp/Si/PW91/csi_e8ag4.pseudo")


# -------- configurations -------- #

# ucell:  unit is A 
ucell = BravaisLattice([
			[ 0, 2.714965, 2.714965  ],
			[ 2.714965, 0, 2.714965  ],
			[ 2.714965, 2.714965, 0  ] ])

sim.atoms = ListOfAtoms([
	Atom( type=Si, position=Vector([ 0, 0, 0 ], ucell) ),
	Atom( type=Si, position=Vector([ 0.25, 0.25, 0.25 ], ucell) )],
	unitcell=ucell)

# -------- input -------- #

sim.encut = PlaneWaveCutoff( 300.00)    # unit is eV
sim.eband = ElectronicBands(20)	# change: electronsperatom*numberofatoms/2+10
sim.vxc = NetCDF.Entry( name="ExcFunctional", value="PW91")	# options: PZ VWN PW91 PBE revPBE RPBE
sim.sym = NetCDF.Entry( name="UseSymmetry", value="Maximum")	# options: Off Maximum

# -------- output -------- #

# sim.debug = NetCDF.Entry( name= "PrintDebugInfo", value="MediumLevel") 	# options: Off MediumLevel HighLevel #
# sim.netcdf = NetCDF.Entry( name="NetCDFOutputControl") #
# sim.netcdf.PrintEffPotential = "Yes" #
# sim.netcdf.PrintElsPotential = "Yes" #
# sim.netcdf.PrintChargeDensity = "Yes" #
# sim.pdos = NetCDF.Entry( name="PrintAtomProjectedDOS") #
# sim.pdos.EnergyWindow = (-23., 1.)   # unit is eV  #
# sim.pdos.EnergyWidth = 0.30          # unit is eV  #
# sim.pdos.NumberEnergyPoints = 100 #
# sim.pdos.CutoffRadius = 1            # unit is A #

# -------- some default parameters -------- #

sim.kpt = NetCDF.Entry( name="KpointSetup", value=[7,7,7])
sim.kpt.gridtype = "MonkhorstPack"	# options: MonkhorstPack ChadiCohen

# sim.eminimization = NetCDF.Entry( name="ElectronicMinimization")
# sim.eminimization.Method = "eigsolve"	# options: resmin eigsolve rmm-diis
# sim.eminimization.DiagonalizationsPerBand = 2
# sim.extracharge = NetCDF.Entry( name="ExtraCharge", value = 0.)	# unit is electrons / unit cell
# sim.dyn = NetCDF.Entry("Dynamics")
# sim.dyn.Type = "Relaxation"	# options: Static Relaxation ElasticBand ExternalIonMotion
# sim.dyn.Method = "Quickmin_Atomwise"	# options: Verlet Quickmin_Atomwise Quickmin_Sum BFGS ConjugateGradient
# sim.dyn.Step = 2.0	# unit is femto-second

# -------- execute -------- #

sim.Execute(outfile="Si-diamond.nc", ascii="Si-diamond.txt")
sim.UpdateFromNetCDFFile("Si-diamond.nc")
sim.all = ListOfEigenStates()

# -------- terminal information -------- #

# print "Total potential energy = ", sim.atoms.GetTotalPotentialEnergy() #
# print "Forces on atoms        = ",  sim.atoms.GetCartesianForces() #
