% CombineUnoccupiedBVSpin.m %
% Bug: %
% ============================================================== %
% Jul. 21, 2005:  %
% This is a critical bug %
% matrix(:,ff) = BlochBasis{ibz,ispin,ff}.orbS(1:homo+GrabDim)'; %
% different atoms can have different orbital numbers,         %
% then when forming "matrix", it needs actual orbital dimension  %
% not the maxima as QOrbMax           %

function [QMB, PsiCoeff, BlochBasis, PW, bsb, bb] = ...
      CombineUnoccupiedBVSpin(Eshift, condmax, ntotalband, Atom, Psp, Bloch, PW)
  
  fprintf (1, '[CombineUnoccupiedBVSpin]  Entering CombineUnoccupiedSpace ...\n');
  
  atomMax    = Atom.AtomMax;
  atomtype   = Atom.Atom2Type;
  atom_phase = Atom.Phase;
  
  TypeMax = Psp{1}.TypeMax;
  IBZKMax = PW.nIBZKs;
  IBZKWeight = PW.IBZKWeight;
  number_of_spin  = PW.nSpins;
  number_of_bands = PW.nBands;
  npl    = PW.nWaves;
  Enk    = PW.EigenValues;
  Efermi = PW.Efermi;
  volume = PW.Volume;
  OccupationNumbers = PW.Occupation;
  
  if (ntotalband>0),
    number_of_bands = ntotalband;
  end;
  
  totalorbital = 0;
  
  for iatom = 1 : atomMax,
    type = atomtype(iatom);
    totalorbital = totalorbital + Psp{type}.orbMax;
  end;
  
  if (number_of_bands<totalorbital),
    fprintf (1, '\n[CombineUnoccupiedBVSpin]  Error: #_of_bands < #_of_orbitals\n');
    pause;
    return;
  end;
  
  
  PW.nOrbitals = totalorbital;
  
  QOrbMax = Psp{1}.orbMax;
  for itype = 1 : TypeMax,
    if (Psp{itype}.orbMax > QOrbMax) ,
      QOrbMax = Psp{itype}.orbMax; 
    end;
  end;
  
  
  PsiCoeff = zeros(IBZKMax, number_of_spin, number_of_bands, atomMax, QOrbMax);
  
  AvgOccupiedBand = 0;
  for ibz = 1 : IBZKMax,
    for ispin = 1 : number_of_spin,
      AvgOccupiedBand = AvgOccupiedBand + ...
          sum(OccupationNumbers(ibz,ispin, :));
    end;
  end;
  
  Asum = 0;
  AvgOccupiedBand = ...
      round( AvgOccupiedBand / 2 / IBZKMax + 0.5 );
  
  AvgGrabDim = totalorbital - AvgOccupiedBand;
  
  fprintf(1,'Total Bands        = %d\n',number_of_bands);
  fprintf(1,'Total Orbitals     = %d\n',totalorbital);
  fprintf(1,'Avg Occupied Band  = %d\n',AvgOccupiedBand);
  fprintf(1,'Avg GrabDim = TotalOrbitals - AvgOccupiedBand = %d\n', AvgGrabDim);
  
  BlochBasis = Bloch;
  
  fprintf(1, '\n');
  fprintf(1, 'number_of_bands = %d\n', number_of_bands );
  fprintf(1, 'IBZKMax         = %d\n', IBZKMax );
  fprintf(1, 'number_of_spin  = %d\n', number_of_spin );
  fprintf(1, '\n');
  
  if ( totalorbital > number_of_bands ),
    fprintf(1, ['Unphysical problem: Total Orbitals > Total Bands. \n\n']);
    return;
  end;
  
  
  GrabDim4IBZ = zeros(IBZKMax, number_of_spin);
  
  % Pt atom: 9 orbitals: s px py pz d1 d2 d3 d4 d5 %
  % H atom:  1 orbital : s%
  % total orbital = 10 %
  % total_bands = 40  %
  % Avg Occupied bands: (10+1)/2 ~= 6 %
  % Avg Unoccupied bands: 40 - 6 = 34 %
  % GrabDim = 10- 6 = 4 %
  % |AO(iatom, iorb)> project onto |psi_nk> = <psi_nk|AO(iatom, iorb)> %
  % 1. |\psi_nk(occupied)> --> <psi_nk(occuiped)|AO(iatom, iorb)> %
  %    6 occupied --> 10 orbitals , no transformation after that%
  %  %
  % 2. |\psi_nk(unoccupied)> --> <psi_nk(unoccuiped)|AO(iatom, iorb)> %
  %   34 unocc --> 10 orbitals:  <psi_nk(unoccuiped)|AO(iatom, iorb)> %
  %   do transformation: rotate 34 |\psi_nk(unoccupied)> to form another %
  %                      34 |Virtual_nk(unoccupied)> %
  %under the condition that they have maximal toatl overlap with Atomic orbitals%
  % That means they have maximally overlap with atomic orbitals %
  % This is achevied by least square method %
  % for each spin and each k point, we will need maximally overlap between all unoccupied bands and all atomic orbitals %
  % after that, we take AA = A^T*A --> (34x34), diagonalize and take the highest 4eigenvalues and corresponding of eigenvectors which is coefficients of different unoccupied bands, these 4 unoccupied and the other 6 occupied bands form another subspace within Bloch space. %
  % Then my quambo is formed as: %
  % |QO(iatom,iorb)> = \sum_(vk in 10 virtual bands in subspace) %
  %                  = |\psi_vk> * <\psi_vk|AO(iatom,iorb)>  %
  % <\psi_vk|AO(iatom,iorb)> = ... %
  %   eigenvector(vk, 1 : unoccupied_nk) * <psi_nk(unoccuiped)|AO(iatom, iorb)>%
  % |QO(iatom,iorb)> is my quambo %
  % Then it gives you maximally localization with respect to atomic orbitals %
  
  
  for ibz = IBZKMax : -1 : 1,
    %   fprintf(1,'%d ',ibz); %
    
    for ispin = 1 : number_of_spin,
    fprintf(1,'ibz= %d,  ispin = %d\n',ibz, ispin);
    
    % cache %
    SBlochData = zeros(number_of_bands, npl);
    for jj = 1 : number_of_bands,
      % S|nk> %
      SBlochData(jj,:) = QuickSBloch(ibz, Bloch{ibz, ispin, jj}.a, ...
                                         Psp, atomtype, atomMax, volume, atom_phase);
      tmptmpwave1 = conj(Bloch{ibz, ispin, jj}.a);
      tmptmpwave2 = SBlochData(jj,:);
      bsb(ispin, ibz, jj) = sum(tmptmpwave1(:).*tmptmpwave2(:))*sqrt(volume);
      bb(ispin, ibz, jj) = sum(conj(tmptmpwave1(:)).*tmptmpwave1(:))*sqrt(volume)^2;
    end;
    
    
    lumo = min( find( Enk(ibz,ispin, :) > ...
                      (Efermi+Eshift) ) );
    homo = lumo - 1;
    
    if (homo>totalorbital),
      homo = totalorbital;
    end;
    
    % # of band for homo level will change for different k point, % 
    % therefore GrabDim will change %
    GrabDim = totalorbital - homo ;
    
    B = zeros(totalorbital, homo);
    for jj = 1 : homo,
      idx = 1;
      for iatom = 1 : atomMax,
        type = atomtype(iatom);
        for iorb = 1 : Psp{type}.orbMax,
          B(idx, jj) = ...
              QuickOrbSBlochBV(Psp{type}.orbK(iorb, :, ibz), ...
                                   atom_phase(iatom, :, ibz), ...
                                   SBlochData(jj,:), volume);
          idx = idx + 1;
        end;
      end;
    end;
    
    [BU,BS,BV] = svd(B);
    BUOCC = BU(:,1:homo);
    BUUNOCC = BU(:,homo+1:size(BU,2));
    
    A = zeros( totalorbital, number_of_bands-homo);
    %  <AO(iatom, iorb) | S | unoccupied_nk>    %
    for jj = 1 : number_of_bands-homo,
      idx = 1;
      for iatom = 1 : atomMax,
        type = atomtype(iatom);
        for iorb = 1 : Psp{type}.orbMax,
          iband = homo + jj;
          % <orb|S|jj> %
          A(idx, jj) = ...
              QuickOrbSBlochBV( Psp{type}.orbK(iorb, :, ibz), ...
                                    atom_phase(iatom, :, ibz), ...
                                    SBlochData(iband,:), volume );
          idx = idx + 1;
        end;
      end;
      A(:, jj) = A(:, jj) - BUOCC * (BUOCC' * A(:, jj));
      
    end;
    
    
    AA = zeros(number_of_bands-homo, number_of_bands-homo);
    for jjp = 1 : number_of_bands-homo,
      for jj = 1 : number_of_bands-homo,
        % \sum_orb <jjp|S|orb><orb|S|jj> 
        AA(jjp,jj) = A(:, jjp)' * A(:, jj);
      end;
    end;
    
    % max_a \sum_orb <\sum_jjp(a_jjp*jjp)|S|orb><orb|S|\sum_jj(a_jj*jj)> = max_a(a'*AA*a) 
    
    [Vold,D] = eig(AA);
    V = Vold;
    
    % For Occupied bands,(1--homo), project these bands to pseudo-orbs %
    for iatom = 1 : atomMax,
      type = atomtype(iatom);
      for iorb = 1 : Psp{type}.orbMax,
        for iband = 1 : homo,
          BlochBasis{ibz,ispin,iband}.orbS(iatom,iorb) = ...
              conj( QuickOrbSBlochBV( Psp{type}.orbK(iorb, :, ibz), ...
                                          atom_phase(iatom, :, ibz), ...
                                          SBlochData(iband,:) , volume) );
          
          PsiCoeff(ibz, ispin, iband, iatom, iorb) = ...
              BlochBasis{ibz,ispin,iband}.orbS(iatom,iorb);
        end;
      end;
    end;
    
    for GrabDim = totalorbital-homo : -1 : 0,
      
      PsiCoeff(ibz, ispin, homo+(1: (number_of_bands-homo) ), :, :) = 0.;
      % |ii> = \sum_jj ( a_jj * |jj> ) %
      % <jj|S|ii> = a_jj = V(jj, number_of_bands-homo+1-ii) %
      
      % For Unoccupied bands,(homo+1 -- homo+GrabDim), % 
      % project to pseudo-orbs with certain number of virtual bands np < n_unoccupied %
      for ii = 1 : GrabDim,
        
        BlochBasis{ibz,ispin,homo+ii}.a = BlochBasis{ibz,ispin,homo+ii}.a * 0;
        BlochBasis{ibz,ispin,homo+ii}.orbS = zeros(atomMax, QOrbMax);
        for jj = 1 : number_of_bands-homo,
          BlochBasis{ibz,ispin,homo+ii}.a = ...
              BlochBasis{ibz,ispin,homo+ii}.a + ...
              V(jj, number_of_bands-homo+1-ii) * Bloch{ibz,ispin,homo+jj}.a;
          
          idx = 1;
          for iatom = 1 : atomMax,
            type = atomtype(iatom);
            for iorb = 1 : Psp{type}.orbMax,
              
              % compress all unoccupied bands to these GrabDim bands %
              BlochBasis{ibz,ispin,homo+ii}.orbS(iatom,iorb) = ...
                  BlochBasis{ibz,ispin,homo+ii}.orbS(iatom,iorb) + ...
                  conj( V(jj, number_of_bands-homo+1-ii) * ...
                        A(idx, jj) );
              
              % <jj|S|quambo> = <jj|S|ii><ii|S|quambo> %
              % <ii|S|quambo> = <ii|S|orb> 
              %               = \sum_jjp <ii|S|jjp><jjp|S|orb> 
              PsiCoeff(ibz, ispin, homo+jj, iatom, iorb) = ...
                  PsiCoeff(ibz, ispin, homo+jj, iatom, iorb) + ...
                  V(jj, number_of_bands-homo+1-ii) * ...
                  squeeze(V(1:number_of_bands-homo, number_of_bands-homo+1-ii))' * ...
                  squeeze(A(idx, 1:number_of_bands-homo))';
              idx = idx + 1;
            end;
          end;
        end;
        % <orb|S|Bloch_{nk}> --> <Bloch_{nk}|S|orb> %
      end;
      
      matrix = zeros(homo+GrabDim);
      for ff = 1 : homo+GrabDim,
        id = 0;
        for iatom = 1:atomMax,
          type = atomtype(iatom);
          matrix( id + (1:Psp{type}.orbMax), ff) = ...
              BlochBasis{ibz,ispin,ff}.orbS(iatom, 1:Psp{type}.orbMax )';
          id = id + Psp{type}.orbMax;
          
          % This is a critical & stupid bug %
          % matrix(:,ff) = BlochBasis{ibz,ispin,ff}.orbS(1:homo+GrabDim)'; %
          % different atoms can have different orbital numbers,         %
          % then when forming "matrix", it needs actual orbital dimension  %
          % not the maxima as QOrbMax           %
          
        end;
      end;
      
      
      fprintf (1,'\tGrabDim=%d: cond=%g\n', GrabDim, cond(matrix));
%       abs(matrix) %
%       pause; %
      
      if ( cond(matrix) < condmax ),
        break;
      else
        % reduce subspace from 10 to 9 %
        % from 6(occ) + 4 (unocc) to  6(occ) + 3 (unocc) = 9  %
        %  clear the GrabDim data which may contaminate QMB2XSF    
        BlochBasis{ibz,ispin,homo+GrabDim}.a = BlochBasis{ibz,ispin,homo+GrabDim}.a * 0;
        BlochBasis{ibz,ispin,homo+GrabDim}.orbS = zeros(atomMax, QOrbMax);
      end;
      
      
    end; % for GrabDim
    
    if (GrabDim == 0),
      fprintf (1,'\n==>  GrabDim=0\npause...\n');
      pause;
    end;
    
    GrabDim4IBZ(ibz, ispin) = GrabDim+homo;
    
    % for other bands, (homo+GrabDim+1 -- number_of_bands), throw away %
    for iatom = 1 : atomMax,
      type = atomtype(iatom);
      for iband = homo+GrabDim+1 : number_of_bands,
        BlochBasis{ibz,ispin,iband}.orbS(iatom,1:Psp{type}.orbMax) = 0;
      end;
    end;
    
    
  end;
end;




% Normalization for QUAMBOs %
% QuamboNorm = D_{i\alpha}^(1/2) in QUAMBO construction %

clear QuamboNorm QuamboDeviation QuamboEnergy;
QuamboNorm      = zeros(atomMax, QOrbMax, number_of_spin);
QuamboDeviation = zeros(atomMax, QOrbMax, number_of_spin);
QuamboEnergy    = zeros(atomMax, QOrbMax, number_of_spin);

for iatom = 1 : atomMax,
  type = atomtype(iatom);
  for iorb = 1 : Psp{type}.orbMax,
    for ispin = 1: number_of_spin,
      for ibz = 1 : IBZKMax,
        for iband = 1 : number_of_bands,
          QuamboNorm(iatom, iorb, ispin) = ...
              QuamboNorm(iatom, iorb, ispin) + ...
              IBZKWeight(ibz) * ...
              conj(PsiCoeff(ibz, ispin, iband, iatom, iorb) ) * ...
              PsiCoeff(ibz, ispin, iband, iatom, iorb);
        end;
      end;
      
      QuamboNorm(iatom, iorb, ispin) = sqrt( real(QuamboNorm(iatom, iorb, ispin)) );
      QuamboDeviation(iatom, iorb, ispin) = sqrt( 2 * abs(1 - QuamboNorm(iatom, iorb, ispin)) );
      
    end;
  end;
end;

for iatom = 1 : atomMax,
  type = atomtype(iatom);
  for iorb = 1 : Psp{type}.orbMax,
    for ibz = 1 : IBZKMax,
      for ispin = 1 : number_of_spin,
        for iband = 1 : totalorbital,
          BlochBasis{ibz,ispin,iband}.orbS(iatom,iorb) = ...
              BlochBasis{ibz,ispin,iband}.orbS(iatom,iorb) / ...
              QuamboNorm(iatom, iorb, ispin);
        end;
      end;
    end;
  end;
end;

for iatom = 1 : atomMax,
  type = atomtype(iatom);
  for iorb = 1 : Psp{type}.orbMax,
    for ispin = 1 : number_of_spin,
      for ibz = 1 : IBZKMax,
        for iband = 1 : number_of_bands,
          PsiCoeff(ibz, ispin, iband, iatom, iorb) = ...
              PsiCoeff(ibz, ispin, iband, iatom, iorb) / ...
              QuamboNorm(iatom, iorb, ispin);
        end;
      end;
    end;
  end;
end;
oldPsiCoeff = [];


for iatom = 1 : atomMax,
  type = atomtype(iatom);
  for iorb = 1 : Psp{type}.orbMax,
    for ispin = 1 : number_of_spin,
      for ibz = 1 : IBZKMax,
        for iband = 1 : number_of_bands,
          QuamboEnergy(iatom, iorb, ispin) = ...
              QuamboEnergy(iatom, iorb, ispin) + ...
              Enk(ibz, ispin, iband) * IBZKWeight(ibz) ...
              * PsiCoeff(ibz, ispin, iband, iatom, iorb);
        end;
      end;
    end;
  end;
end;

QMB.QuamboNorm      = QuamboNorm;
QMB.QuamboEnergy    = QuamboEnergy;
QMB.QuamboDeviation = QuamboDeviation;
QMB.GrabDim4IBZ     = GrabDim4IBZ;
QMB.number_of_bands = number_of_bands;

fprintf (1, '\n\n');

return;

