% ConvertPsp.m %
% Convert from A.U. system in pseudopotential file %
% to Angstrom / eV system in Dacapo %

function [ Psp ] = ConvertPsp(Psp, ralpha, plotflag, add_del_orb, tmpsavedir);
  
  fprintf (1, '[ConverPsp]  Convert units of ultrasoft pseudopotential\n');
  units;
  
  TypeMax = Psp{1}.TypeMax;
  
  for type = 1 : TypeMax,
    Psp{type}.RadialGrid   = Psp{type}.RadialGrid   * BOHR_RADIUS_IN_A;
    Psp{type}.RadialGridAB = Psp{type}.RadialGridAB * BOHR_RADIUS_IN_A;
    Psp{type}.phi_E  = Psp{type}.phi_E * RYDBERG_IN_EV;
    Psp{type}.rphi   = Psp{type}.rphi / sqrt(BOHR_RADIUS_IN_A);
    Psp{type}.rbeta  = Psp{type}.rbeta / sqrt(BOHR_RADIUS_IN_A);
    %   Psp{type}.rgriddiff = Psp{type}.RadialGridAB; %
    Psp{type}.rgriddiff = Psp{type}.rgriddiff * BOHR_RADIUS_IN_A;
    
    Psp{type}.rphiOLD = Psp{type}.rphi;
    rgrid = Psp{type}.RadialGrid;
    
    for iphi = 1 : Psp{type}.phiMax,
      
      rphinormOLD(type, iphi) = ...
          HLMSHLM( Psp, Psp{type}.ngrid, Psp{type}.rgriddiff,  ...
                   Psp{type}.rphi(iphi,:), Psp{type}.phi_L(iphi), 0, ...
                   type, ...
                   Psp{type}.rphi(iphi,:), Psp{type}.phi_L(iphi), 0 );
      
      if ( (abs(ralpha(Psp{type}.phi_L(iphi)+1) ) > 1e-6) || (add_del_orb==1)),
        rexp = exp(- ralpha(Psp{type}.phi_L(iphi)+1) * rgrid);
        Psp{type}.rphi(iphi, :) = Psp{type}.rphi(iphi, :) .* rexp(1,:) ;
        % rescale back to norm = 1  %
        r2scale = HLMSHLM( Psp, Psp{type}.ngrid, Psp{type}.rgriddiff, ...
                           Psp{type}.rphi(iphi,:), Psp{type}.phi_L(iphi), 0, ...
                           type, ...
                           Psp{type}.rphi(iphi,:), Psp{type}.phi_L(iphi), 0 );
        Psp{type}.rphi(iphi, :) = 1/sqrt(r2scale) * Psp{type}.rphi(iphi, :);
        
        % double check norm = 1?  %
        rphinorm(type, iphi) = ...
            HLMSHLM( Psp, Psp{type}.ngrid, Psp{type}.rgriddiff,  ...
                     Psp{type}.rphi(iphi,:), Psp{type}.phi_L(iphi), 0, ...
                     type, ...
                     Psp{type}.rphi(iphi,:), Psp{type}.phi_L(iphi), 0 );
      else
        rphinorm(type, iphi) = rphinormOLD(type, iphi);
      end;
      
      
    end;
    
    
    if (plotflag==1),
      hfig = figure('visible', 'off');
      plot(Psp{type}.RadialGrid, Psp{type}.rphi(1:Psp{type}.phiMax,:),'-');
      hold on;
      plot(Psp{type}.RadialGrid, Psp{type}.rphiOLD(1:Psp{type}.phiMax,:),'o');
      drawnow;
      print('-depsc', '-r300',  [tmpsavedir '/Quambo.atomicorbital.eps']);
      close(hfig);
    end;
        
  end;
  
  return;
  