% LoadDACAPO.m %

function [Atom, PW, Bloch] = LoadDACAPO(dft_outfile_nc, Psp)
  
 
  ResultNetCDF = netcdf(dft_outfile_nc, 'nowrite');
  
  TypeMax = Psp{1}.TypeMax;
  
  number_IBZ_kpoints = size(ResultNetCDF('number_IBZ_kpoints'),1);


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  fprintf (1, '\n');
  fprintf (1, '[LoadDACAPO]  Load DACAPO atomic information\n');

  Atom.TypeMax = TypeMax;
  DynamicAtomSpecies = ResultNetCDF{'DynamicAtomSpecies'}(:);
  Atom.AtomMax = size(DynamicAtomSpecies,1);

%  for DACAPO  %
  for iatom = 1 : Atom.AtomMax,
    if DynamicAtomSpecies(iatom,1) == ' '
      Atom.AtomSymbol{iatom} = DynamicAtomSpecies(iatom,2);
    else
      Atom.AtomSymbol{iatom} = DynamicAtomSpecies(iatom,1:2);
    end;
    Atom.Atom2Type(iatom) = strmatch(Atom.AtomSymbol{iatom}, Psp{1}.SymbolList);
  end;

%  for VASP  %
%   for iatom = 1 : Atom.AtomMax, %
%     itype = DynamicAtomSpecies(iatom, 2); %
%     Atom.AtomSymbol{iatom} = Psp{itype}.Symbol; %
%     Atom.Atom2Type(iatom)  = itype; %
%   end; %
  
  for itype = 1 : Atom.TypeMax,
    Atom.Type2Atom{itype} = find(Atom.Atom2Type == itype);
  end;
  
  UnitCell = ResultNetCDF{'UnitCell'};
  Atom.NumberIonicSteps = size(UnitCell,1);
  Atom.H = UnitCell(Atom.NumberIonicSteps,:,:); 
  Atom.Volume = abs(det(Atom.H));
  
  Atom.G = 2 * pi * inv(Atom.H);
  
  DynamicAtomPositions = ResultNetCDF{'DynamicAtomPositions'};
  for iatom = 1 : Atom.AtomMax,
    Atom.S(iatom,:) = DynamicAtomPositions(Atom.NumberIonicSteps,iatom,:);
    Atom.X(iatom,:) = Atom.S(iatom,:) * Atom.H;
  end;
  


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  
  
  fprintf (1, '\n');
  fprintf (1, '[LoadDACAPO]  Load DACAPO Bloch wavefunction\n');
  
  number_IBZ_kpoints = size(ResultNetCDF('number_IBZ_kpoints'),1);
  number_plane_waves = size(ResultNetCDF('number_plane_waves'),1);
  
  NumberIonicSteps = size(UnitCell,1);
  PW.Volume   = abs(det(Atom.H));
  G           = Atom.G;
  IBZK        = ResultNetCDF{'IBZKpoints'}(:);
  
  PW.IBZK     = IBZK;
  PW.IBZKMax  = size(PW.IBZK,1);
  
  IBZKWeight     = ResultNetCDF{'KpointWeight'}(:);
  PW.IBZKWeight  = IBZKWeight/sum(IBZKWeight);
  KpointSetup    = ResultNetCDF{'KpointSetup'}(:);
  PW.KpointSetup = KpointSetup';
  BZKMax         = KpointSetup(1) * KpointSetup(2) * KpointSetup(3);
  PW.BZKMax      = BZKMax;
  [BZK]          = Generate_BZK(PW.KpointSetup);
  PW.BZK         = BZK;
  
  tmpWaveFunction = ResultNetCDF{'WaveFunction'};
  tmpWave         = ResultNetCDF{'WaveFunction'}(:);
  PW.nIBZKs = size(tmpWaveFunction,1);
  PW.nSpins = size(tmpWaveFunction,2);
  PW.nBands = size(tmpWaveFunction,3);
  PW.nWaves = size(tmpWaveFunction,4);
  
  WaveFunction = reshape(tmpWave, ...
                         PW.nIBZKs, PW.nSpins, PW.nBands, PW.nWaves, 2);
  tmpWaveFunction = [];
  
  tmpEigenValues  = ResultNetCDF{'EigenValues'};
  tmpEigen        = ResultNetCDF{'EigenValues'}(:);
  PW.nIonicSteps  = size(tmpEigenValues,1);
  EigenValues     = reshape(tmpEigenValues(PW.nIonicSteps,:,:,:), PW.nIBZKs, PW.nSpins, PW.nBands);
  
  PW.EigenValues  = EigenValues;
  
  Bloch = cell(PW.IBZKMax, PW.nSpins, PW.nBands);
  
  for ibz = 1 : PW.IBZKMax,
    for ispin = 1 : PW.nSpins,
      for iband = 1 : PW.nBands,
        Bloch{ibz,ispin,iband}.a = ...
            ( WaveFunction(ibz,ispin,iband,:,1) + ...
              j * WaveFunction(ibz,ispin,iband,:,2) ) / sqrt(PW.Volume);
        Bloch{ibz,ispin,iband}.a = Bloch{ibz,ispin,iband}.a(:);
      end;
    end;
  end;
  
  tmpOccupation   = ResultNetCDF{'OccupationNumbers'};
  PW.Occupation   = reshape(tmpOccupation(PW.nIonicSteps,:,:,:), PW.nIBZKs, PW.nSpins, PW.nBands);
  
  EnergyCutOff    = ResultNetCDF{'PlaneWaveCutoff'}(:);
  PW.EnergyCutOff = EnergyCutOff(:);
  Efermi    = ResultNetCDF{'FermiLevel'}(:);
  PW.Efermi = Efermi;
  
  FFTIndex = ResultNetCDF{'WaveFunctionFFTindex'}(:);
  NGX = size(ResultNetCDF('softgrid_dim1') ,1);
  NGY = size(ResultNetCDF('softgrid_dim2') ,1);
  NGZ = size(ResultNetCDF('softgrid_dim3') ,1);
  PW.nGrid = [NGX NGY NGZ];
  NG = PW.nGrid;

% for DACAPO, it is already ok for use. %  
  PW.FFTIndex = reshape(FFTIndex, PW.IBZKMax, 3, PW.nWaves);

% for VASP, it is one collum of index, need to reform   %
%   for ibz = 1 : PW.IBZKMax, %
%     f=FFTIndex(ibz,:); %
%     x= mod(f,NGX ); %
%     indx = find(x==0); %
%     x(indx) = NGX; %
%     y= mod(f-x, NGY*NGX)/NGX +1; %
%     z= (f-x-(y-1)*NGX)/(NGY*NGX) +1; %
%     indf = [x' y' z']; %
%     PW.FFTIndex(ibz,:,:) = indf'; %
%   end; %

  PW.IBZGmin = ResultNetCDF{'NumberPlaneWavesKpoint'}(:);
  
  
  
  PW.oldIBZK    = PW.IBZK;
  PW.oldIBZKMax = PW.IBZKMax;
  PW.oldIBZKWeight = PW.IBZKWeight;

  PW.oldIBZK  = PW.oldIBZK * G';  
  PW.IBZK     = PW.IBZK * G';
  PW.BZK      = PW.BZK * G';
  
  
  % ROTATE WAVEFUNCTION in IBZ TO GET WAVEFUNCTION IN FULL BZ %
  nLatRot = size(ResultNetCDF{'LatticeRotationMatrix'}, 1);
  LatRot = ResultNetCDF{'LatticeRotationMatrix'}(:);
  if (PW.BZKMax> round(2*PW.IBZKMax-1) ),
    numspa = nLatRot;
    
    for ispa = 1: numspa,
       % for DACAPO, no tranpose is needed for LatRot %
      LatRot(ispa,:,:)  = squeeze(LatRot(ispa,:,:));
      CartRot(ispa,:,:) = G * squeeze(LatRot(ispa,:,:)) * inv(G);
    end;
    PW.LatRot  = LatRot;
    PW.CartRot = CartRot;
    
    [PW, Bloch, Psp] = QuickWavefunction(PW, Bloch, Atom.G, Atom.H, Psp); 
   
  end;
  
  
  close(ResultNetCDF);
  
  
  return;
  
