% LoadPsp.m %
% 1. Read from PspList.txt the pseudopotential file list    %
% 2. Convert them to netcdf format using binary2netcdf utility %
%    (http://listserv.fysik.dtu.dk/pipermail/campos/2003-August/000633.html) %
% 3. Load netcdf files %


function [Psp, iflag_success] = LoadPsp(Input, iFlagBigendian);

  iflag_success = 1;
  
  fprintf (1, '\n');
  fprintf (1, '[LoadPsp]  Load DACAPO ultrasoft pseudopotential\n');
  elementsymbol;
  
  tmpdatadir = Input.DFTDataDir;
  addorbital = Input.ConvertPsp.AddOrDeleteOrbital;
  if (addorbital==1),
    addorbital_L = Input.ConvertPsp.AddOrbital_L;
  elseif (addorbital==-1),
    deleteorbital = Input.ConvertPsp.DeleteOrbitalIndex;
  end;
  
  
  nPsp = round(length(Input.PseudopotList)/2);
  Psp{1}.TypeMax = nPsp;
  
  for ipsp = 1 : nPsp,
    PspSymbolList{ipsp} = Input.PseudopotList{2*ipsp-1};
    psppath{ipsp}       = Input.PseudopotList{2*ipsp};
  end;
  
  strb2n = which('binary2netcdf.x');
  if isempty(strb2n),
    fprintf(1, ['[LoadPsp]  Can not find "binary2netcdf.x".\n']);
    iflag_success = 0;
    return;
  end;
  
  
    
  TypeMax = length(PspSymbolList);
  
  for type = 1 : TypeMax,
    fprintf (1, '%s uses %s,\n', PspSymbolList{type}, psppath{type});
    
    fn_PspNetCDF = [tmpdatadir PspSymbolList{type} '.nc'];
    fn_PspTxt = [tmpdatadir PspSymbolList{type} '.txt'];
    
    system(['setenv F_UFMTENDIAN big; ' strb2n ' '...
            psppath{type} ' ' fn_PspNetCDF ' > ' fn_PspTxt]);
    fprintf (1, 'which is converted to NetCDF file %s,\n', fn_PspNetCDF);
    fprintf (1, 'with brief summary in %s.\n', fn_PspTxt);
    PspNetCDF = netcdf(fn_PspNetCDF, 'nowrite');
    fprintf(1, '\n');
    
    Psp{type}.AtomicNumber = strmatch(PspSymbolList{type},...
                                      ElementSymbol,'exact');
    Psp{type}.Symbol = PspSymbolList{type};
    
    Psp{type}.RadialGrid   = PspNetCDF{'RadialGrid'}(:)';
    Psp{type}.mesh         = length(Psp{type}.RadialGrid);
    Psp{type}.RadialGridAB = PspNetCDF{'RadialGridDiff'}(:)';  
    
    Psp{type}.ngrid = 2*(floor(Psp{type}.mesh/2)+1)-1;
    Psp{type}.rgriddiff = Psp{type}.RadialGridAB(1,1:Psp{type}.ngrid);
    
    Psp{type}.phiType = PspNetCDF{'OrbitalType'}(:);
    Psp{type}.phiMax  = length(Psp{type}.phiType);
    Psp{type}.phi_L   = floor( mod(Psp{type}.phiType, 100) / 10 );
    Psp{type}.phi_E   = PspNetCDF{'OrbitalEnergy'}(:);
    Psp{type}.rphi    = PspNetCDF{'Orbital_RadialFunction'}(:);
    
    
    % add additional orbitals %
    if (addorbital == 1),
      for iLL = 1 : length(addorbital_L),
        Psp{type}.phi_L = [Psp{type}.phi_L' addorbital_L(iLL) ];
        aa    = []; 
        tmpa  = [];
        aa    = Psp{type}.rphi;
        tmpa  = [aa' aa(1,:)'];
        Psp{type}.rphi   = [];
        Psp{type}.rphi   = tmpa';
        Psp{type}.phiMax = Psp{type}.phiMax + 1;
      end;
      
    elseif (addorbital == -1),
      Psp{type}.newphi_L = [];
      Psp{type}.newrphi  = [];
      for iLL = 1 : length(deleteorbital),
        for iphi = 1: length(Psp{type}.phi_L),
          if (iphi == deleteorbital(iLL)),
            Psp{type}.phiMax = Psp{type}.phiMax - 1;
          else
            tmpL = [];
            aa   = [];
            tmpa = [];
            tmpL = [ Psp{type}.newphi_L' Psp{type}.phi_L(iphi)];
            Psp{type}.newphi_L = tmpL';
            aa    = Psp{type}.rphi(iphi,:);
            tmpa  = [Psp{type}.newrphi' aa'];
            Psp{type}.newrphi  = [];
            Psp{type}.newrphi  = tmpa';
          end;
        end;
      end;
      
      Psp{type}.phi_L = [];
      Psp{type}.rphi  = [];
      Psp{type}.phi_L = Psp{type}.newphi_L;
      Psp{type}.rphi  = Psp{type}.newrphi;
    end;
    
    
    iorb = 0;
    for iphi = 1 : Psp{type}.phiMax,
      for M = -Psp{type}.phi_L(iphi) : Psp{type}.phi_L(iphi),
        iorb = iorb + 1;
        Psp{type}.iphi(iorb) = iphi;
        Psp{type}.orb_L(iorb) = Psp{type}.phi_L(iphi);
        Psp{type}.orb_M(iorb) = M;
      end;
    end;
    
    Psp{type}.orbMax = iorb;
    
    Psp{type}.beta_L = PspNetCDF{'BetaProjectors_AngularMomentum'}(:);
    Psp{type}.betaMax = length(Psp{type}.beta_L);
    Psp{type}.rbeta = PspNetCDF{'BetaProjectors_RadialFunction'}(:);
    Psp{type}.kkbeta = length(Psp{type}.rbeta);
    Psp{type}.rbeta(:, Psp{type}.kkbeta+1:Psp{type}.mesh) = ...
        zeros(Psp{type}.betaMax, Psp{type}.mesh-Psp{type}.kkbeta);
    
    Psp{type}.Qbeta    = PspNetCDF{'AugmentationFunctions_IntegratedQnm'}(:);
    Psp{type}.D0      = PspNetCDF{'Dnm_unscreened'}(:);
    Psp{type}.D       = PspNetCDF{'Dnm_screened'}(:);
    Psp{type}.Rinner  = PspNetCDF{'AugmentationFunctions_Rinner'}(:);
    Psp{type}.Qcoeff  = PspNetCDF{'AugmentationFunctions_Coefficients'}(:);
    
    iproj = 0;
    for ibeta = 1 : Psp{type}.betaMax,
      for M = -Psp{type}.beta_L(ibeta) : Psp{type}.beta_L(ibeta),
        iproj = iproj + 1;
        Psp{type}.ibeta(iproj)  = ibeta;
        Psp{type}.proj_L(iproj) = Psp{type}.beta_L(ibeta);
        Psp{type}.proj_M(iproj) = M;
      end;
    end;
    Psp{type}.projMax = iproj;
    
    Psp{type}.Q = zeros(Psp{type}.projMax, Psp{type}.projMax);
    for iproj = 1 : Psp{type}.projMax,
      mbeta = Psp{type}.ibeta(iproj);
      iL = Psp{type}.proj_L(iproj);
      iM = Psp{type}.proj_M(iproj);
      for jproj = 1 : Psp{type}.projMax,
        nbeta = Psp{type}.ibeta(jproj);
        jL = Psp{type}.proj_L(jproj);
        jM = Psp{type}.proj_M(jproj);
        if ((iL==jL) && (iM==jM)),
          Psp{type}.Q(iproj, jproj) = Psp{type}.Qbeta(mbeta, nbeta);
        end;
      end;
    end;
    
    fprintf(1, 'L (quantum number):   ');
    for iLL = 1 : length(Psp{type}.orb_L),
      fprintf(1, '   %d', Psp{type}.orb_L(iLL) );
    end;
    fprintf(1, '\n');
    
    close(PspNetCDF);
    fprintf (1, '\n');
  end;

  Psp{1}.SymbolList =  PspSymbolList;
   
