% QMB2XSF.m %

function [iflag_QMB2MAT] = QMB2XSF(Input, Atom, Psp, PW, BlochBasis)

  format long;
  
  xsfname         = Input.OutputQuambo.OrbitalFilename;
  quambo_filename = Input.OutputQuambo.DataFilename;
  Shell           = Input.OutputQuambo.SizeOfBV;
  LAtomicOrbital  = Input.OutputAtomicOrbital.YesorNo;
  LOutputAllAtom  = Input.OutputQuambo.AllAtom;

  iflag_QMB2MAT = 0;
  atomMax    = Atom.AtomMax;
  atomtype   = Atom.Atom2Type;
  atom_X     = Atom.X;
  H          = Atom.H;
  G          = Atom.G;
  atomsymbol = Atom.AtomSymbol;
  volume     = Atom.Volume;
  IBZK       = PW.IBZK;
  IBZKMax    = PW.nIBZKs;
  WaveFunctionFFTindex = PW.FFTIndex;
  IBZKWeight = PW.IBZKWeight;
  NG         = PW.nGrid;
  number_plane_waves = PW.nWaves;
  number_of_spin = PW.nSpins;
  totalorbital = PW.nOrbitals;
  
  if (LOutputAllAtom == 0),
    AtomList = Input.OutputQuambo.AtomList;
  end;
  
  fid_xsf = fopen( xsfname ,'wt');
  if (fid_xsf<0)
    fprintf(1,'[QMB2MAT] Cannot creat file: %s \n',xsfname);
    return;
  end;
  
  
fprintf(1,'[QMB2MAT] Begin Generating file: %s \n',xsfname);
fprintf(fid_xsf,'ATOMS\n');
for iatom = 1 : atomMax,
  type = atomtype(iatom);
  for icell = 1: Shell(1),
    for jcell = 1: Shell(2),
      for kcell = 1: Shell(3),
        
        atom_XCell(iatom,:) = ...
            atom_X(iatom,:) + ...
            ( (icell-floor(Shell(1)/2+1) )* H(1,:) + ...
              (jcell-floor(Shell(2)/2+1) )* H(2,:) + ...
              (kcell-floor(Shell(3)/2+1) )* H(3,:) );
        
        fprintf(fid_xsf, '%d  %f %f %f\n', ...
                Psp{type}.AtomicNumber, ...
                atom_XCell(iatom,1),atom_XCell(iatom,2),atom_XCell(iatom,3));
      end;
    end;
  end;
end;

  
  
  
ibz = 1;
[GIndex, ingx, ingy, ingz] = GIndexGenerator(WaveFunctionFFTindex,0,1,0);
[s1, s2, s3] = ndgrid(0:ingx-1, 0:ingy-1, 0:ingz-1);
s1 = s1 / ingx;
s2 = s2 / ingy;
s3 = s3 / ingz;
X = s1 * H(1,1) + s2 * H(2,1) + s3 * H(3,1);
Y = s1 * H(1,2) + s2 * H(2,2) + s3 * H(3,2);
Z = s1 * H(1,3) + s2 * H(2,3) + s3 * H(3,3);

if (LOutputAllAtom == 0),
  atomMaxOut  = length(AtomList);
else
  atomMaxOut  = atomMax;
end;

totalorbital = 0;
for iatom = 1 : atomMax,
  type = atomtype(iatom);
  totalorbital = totalorbital + Psp{type}.orbMax;
end;




for indexatom = 1 : atomMaxOut,
  
  if (LOutputAllAtom == 0),
    iatom = AtomList(indexatom);
  else
    iatom = indexatom;
  end;
  
  fprintf(fid_xsf,'BEGIN_BLOCK_DATAGRID_3D\n');  
  type = atomtype(iatom);
  iflag_grid = 0;
  
  fprintf(fid_xsf,'Quambo_Orbital_Grid_Atom_%s_%d\n',...
          atomsymbol{iatom},iatom);
  
  
  for ispin = 1 : number_of_spin,
    
    spintag = [''];
    if ((ispin == 1) && (number_of_spin == 2)),
      spintag = ['_spin_up'];
    end;
    if (ispin == 2  ),
      spintag = ['_spin_down'];
    end;
    
   for iorb = 1 : Psp{type}.orbMax,
    Quambo    = [];
    AtomicOrb = [];
    Quambo    = zeros(Shell(1)*ingx,Shell(2)*ingy,Shell(3)*ingz);
    AtomicOrb = zeros(Shell(1)*ingx,Shell(2)*ingy,Shell(3)*ingz);
    
    iAtomicOrbital = 0;
    
    for ibz = 1 : IBZKMax,
      
      [GIndex, ingx, ingy, ingz] = GIndexGenerator(WaveFunctionFFTindex,0,ibz,0);
      GIndex = GIndex .*(GIndex>0);
      Gmin = round(min(find(GIndex(:)==0)-1));
      
      if isempty(Gmin)
        Gmin=length(GIndex);
      end;

      GCubeAtomicOrbitalReshaped = zeros(1,ingx*ingy*ingz);
      GCubeAtomicOrbital = zeros(ingx, ingy, ingz);
      AtomicOrbitalCube  = zeros(ingx, ingy, ingz);
      GList = zeros(length(WaveFunctionFFTindex(1,1,:)), 3);
      
      GList = GListGenerator(ibz, WaveFunctionFFTindex, NG);
      GList = GList * G';
      
      KList = GList + ones(number_plane_waves,1) * IBZK(ibz,:);
      
      GCubeAtomicOrbitalReshaped(1, GIndex(1:Gmin,1)) = ...
          conj(Psp{type}.orbK(iorb,1:Gmin,ibz)) / volume .* ...
          exp( j * KList(1:Gmin,:) * atom_X(iatom,:)')';
      GCubeAtomicOrbital = reshape(GCubeAtomicOrbitalReshaped,ingx,ingy,ingz);
      
      AtomicOrbitalCube  = ifftn(GCubeAtomicOrbital)*ingx*ingy*ingz .* ...
          exp( j * (X*IBZK(ibz,1) + Y*IBZK(ibz,2) + Z*IBZK(ibz,3)));   
      
      % Here, ifftn(GCubeAtomicOrbital) stores the coefficients of plane-wave,%
      % which means we need to muliply with exp( i (k+G) r) %
      % to get the bloch wave function in 3-D realspace %
      % Dacapo/netCDF p40, 2.3.51 WaveFunction %
      % \phi_nk(r) = exp( i k r) * u_nk(r) %
      
        for iband = 1 : totalorbital,
          
          GCubeReshaped = zeros(1,ingx*ingy*ingz);
          GCube = zeros(ingx, ingy, ingz);
          GCubeReshaped(1,GIndex(1:Gmin,1)) = ...
              BlochBasis{ibz,ispin,iband}.a(1:Gmin).';
          GCube = reshape(GCubeReshaped,ingx,ingy,ingz);
          
          BlochBasisCube = ifftn(GCube)*ingx*ingy*ingz .* ...
              exp( j * (X*IBZK(ibz,1) + Y*IBZK(ibz,2) + Z*IBZK(ibz,3)) );
          
          for  icell = 1: Shell(1),
            for jcell = 1: Shell(2),
              for kcell = 1: Shell(3),
                
                istart = (icell-1)*ingx+1;
                iend   = icell*ingx;
                jstart = (jcell-1)*ingy+1;
                jend   = jcell*ingy;
                kstart = (kcell-1)*ingz+1;
                kend   = kcell*ingz;
                
                % Quambo = sum_{nk} |Bloch_nk> <Bloch_nk| S |Orb> %
                Quambo( istart:iend, jstart:jend, kstart:kend ) = ...
                    Quambo( istart:iend, jstart:jend, kstart:kend ) + ...
                    IBZKWeight(ibz) * ...
                    real( BlochBasisCube * ...
                          exp( j * ( (icell-floor(Shell(1)/2+1))* H(1,:) + ...
                                     (jcell-floor(Shell(2)/2+1))* H(2,:) + ...
                                     (kcell-floor(Shell(3)/2+1))* H(3,:) ) * ...
                               IBZK(ibz,:)') * ...
                          BlochBasis{ibz,ispin,iband}.orbS(iatom,iorb) );
                
                if (iAtomicOrbital == 0) && (LAtomicOrbital ),
                  
                    AtomicOrb( istart:iend, jstart:jend, kstart:kend ) = ...
                        AtomicOrb( istart:iend, jstart:jend, kstart:kend ) + ...
                        IBZKWeight(ibz) * ...
                        real(AtomicOrbitalCube * ...
                             exp( j * ( (icell-floor(Shell(1)/2+1))* H(1,:) + ...
                                        (jcell-floor(Shell(2)/2+1))* H(2,:) + ...
                                        (kcell-floor(Shell(3)/2+1))* H(3,:) ) * ...
                                  IBZK(ibz,:)'));
                end;
                
                
              end; % kshell %
            end; % jshell
          end; % ishell %
          iAtomicOrbital = 1;
         
        end; % iband %
      
      iAtomicOrbital = 0;
    end; % ibz %
    
    % --------------------------------------------------- %
    SH = [ingx ingy ingz] .* Shell;
    s  = [0 0 0];
    Hgeneral = diag((SH-1)./SH .* Shell ) * H; 
    shift_x = s* H + ...
              (1 - floor(Shell(1)/2+1) )* H(1,:) + ...
              (1 - floor(Shell(2)/2+1) )* H(2,:) + ...
              (1 - floor(Shell(3)/2+1) )* H(3,:);
    
    % --------------------Print QUAMBO------------------------------- %
    
    fprintf(fid_xsf,'BEGIN_DATAGRID_3D_Atom_%s_%d_QUAMBO_orb#%d%s\n', ...
            atomsymbol{iatom},iatom,iorb,spintag);
    fprintf(fid_xsf,'%d %d %d\n', SH);
    fprintf(fid_xsf,'%f %f %f\n', shift_x);
    fprintf(fid_xsf,'%f %f %f\n', Hgeneral');    
    
    fprintf(1,'[QMB2MAT] Atom %d -- %s -- %s -- QUAMBO: <orb %d | orb %d> = %f \n', ...
            iatom,atomsymbol{iatom}, spintag, iorb, iorb, ...
            sum(sum(sum(conj(Quambo).*Quambo)))*volume/ingx/ingy/ingz);
    
    id = 0;
    for iz = 1: Shell(3)*ingz,
      for iy = 1: Shell(2)*ingy,
        for ix = 1: Shell(1)*ingx,
          fprintf(fid_xsf,'%f ',real(Quambo(ix,iy,iz)));
          id = id + 1;
          if (id == 8) 
            id =0;
            fprintf(fid_xsf,'\n');
          end;
        end;
      end;
    end;
    fprintf(fid_xsf,'\nEND_DATAGRID_3D\n');
    
    if (LAtomicOrbital),
      % --------------------Print ATOMIC------------------------------- %
      fprintf(fid_xsf,'BEGIN_DATAGRID_3D_Atom_%s_%d_ATOMIC_orb#%d%s\n', ...
              atomsymbol{iatom},iatom,iorb,spintag);
      fprintf(fid_xsf,'%d %d %d\n', SH);
      fprintf(fid_xsf,'%f %f %f\n', shift_x);
      fprintf(fid_xsf,'%f %f %f\n', Hgeneral');    
      
      fprintf(1,'[QMB2MAT] Atom %d -- %s --%s -- ATOMIC: <orb %d | orb %d> = %f \n', ...
              iatom,atomsymbol{iatom}, spintag, iorb, iorb, ...
              sum(sum(sum(conj(AtomicOrb).*AtomicOrb)))*volume/ingx/ingy/ingz);
      
      id = 0;
      for iz = 1: Shell(3)*ingz,
        for iy = 1: Shell(2)*ingy,
          for ix = 1: Shell(1)*ingx,
            fprintf(fid_xsf,'%f ',real(AtomicOrb(ix,iy,iz)));
            id = id + 1;
            if (id == 8) 
              id =0;
              fprintf(fid_xsf,'\n');
            end;
          end;
        end;
      end;    
      fprintf(fid_xsf,'\nEND_DATAGRID_3D\n');
      % --------------------------------------------------- %
    end;
    
    ImagPart = sum(sum(sum(abs(imag(Quambo))))) / sum(sum(sum(abs(Quambo))));
    
    QUAMBO{iatom}.RGrid(:,:,:,iorb,ispin) = Quambo;
    
  end; % iorb %
  
  end; % ispin%

  
  fprintf(fid_xsf,'END_BLOCK_DATAGRID_3D\n');

end; % iatom %

fclose(fid_xsf);
fprintf(1,'[QMB2MAT] Succeed in Generating Quambo Orbital file: %s \n',xsfname);
save(quambo_filename, 'QUAMBO');

fprintf(1,'\n[QMB2MAT] Succeed in Quambo data file in mat format: %s \n',quambo_filename);
  
  
  iflag_QMB2MAT = 1;
  
  return;
  
