% QuickFermiSurface.m %

function [QMB] = QuickFermiSurface(Input, QMB)
  
  fprintf (1, '[QuickFermiSurface]  Calculate tightbinding Fermi surface\n');
  fprintf (1, '\n');
  
  TBHamOverlap    = QMB.TBHamOverlap;
  TBSOverlap      = QMB.TBSOverlap;
  Rdim            = QMB.Rdim;
  Enk             = QMB.Enk_PW;
  number_of_spin  = QMB.nSpins;
  atomMax         = QMB.atomMax;
  atomtype        = QMB.atomtype;
  G               = QMB.G;
  H               = QMB.H;
  Efermi          = QMB.Efermi;
  orbMax          = QMB.orbMax;
  atom_X          = QMB.atom_X;
  
  saveDir         = Input.saveDir;
  GenEigThreshold = Input.GenEig.Threshold;
  NKgrid     = Input.Fermisurface.Grid;
  Lrcutoff   = Input.BandStructure.RCutoffYesOrNo;
  Rcutoff    = Input.BandStructure.RCutoff;
  fname      = Input.Fermisurface.Filename;
  
  
  
  G0 = [0 0 0];
  KX = zeros(3, NKgrid(1));
  KY = zeros(3, NKgrid(2));
  KZ = zeros(3, NKgrid(3));
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  GG = G';
  
  DKX = (GG(1,:) - G0) / (NKgrid(1)-1);
  DKY = (GG(2,:) - G0) / (NKgrid(2)-1);
  DKZ = (GG(3,:) - G0) / (NKgrid(3)-1);
  for ii = 1 : 3,
    if ( abs(DKX(ii)) > 1e-6 ),
      KX(ii,:) = [G0(ii) : DKX(ii) : GG(1,ii)];
    else
      KX(ii,:) = zeros(1,NKgrid(1));
    end;
    
    if ( abs(DKY(ii)) > 1e-6 ),
      KY(ii,:) = [G0(ii) : DKY(ii) : GG(2,ii)];
    else
      KY(ii,:) = zeros(1,NKgrid(2));
    end;
    
    if ( abs(DKZ(ii)) > 1e-6 ),
      KZ(ii,:) = [G0(ii) : DKZ(ii) : GG(3,ii)];
    else
      KZ(ii,:) = zeros(1,NKgrid(3));
    end;
  end;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
  Hdim = 0;
  for iatom = 1:atomMax,
    itype = atomtype(iatom);
    Hdim  = Hdim + orbMax(itype);
  end;
  
  
  id = 0;
  for ixx = -Rdim(1) : Rdim(1),
    ix = ixx + Rdim(1) + 1;
    for iyy = -Rdim(2) : Rdim(2),
      iy = iyy + Rdim(2) + 1;
      for izz = -Rdim(3) : Rdim(3),
        iz = izz + Rdim(3) + 1;
        
        id = id + 1;
        RCellID(id, :)  = [ix iy iz];
        RCellPos(id, :) = [ixx iyy izz];
        
      end;
    end;
  end;
  
  if (Lrcutoff ~= 0),
    for iatom = 1 : atomMax,
      Ratomi = atom_X(iatom,:);
      for jatom = 1 : atomMax,
        idx = 1;
        for id = 1 : size(RCellID,1),
          Ratomj   = RCellPos(id, :)*H + atom_X(jatom,:);
          DRatomij = abs(norm(Ratomj - Ratomi));
          if (DRatomij <= Rcutoff),
            RCellIDTemp(iatom, jatom, idx) = id;
            RCellPosTemp(iatom, jatom, idx, 1:3) = RCellPos(id, :);
            idx = idx + 1;
          end;
        end; % for id = 1 : size(RCellID,1)
        RCellIDMax(iatom,jatom) = idx-1;
      end;
    end;
  end;
  
  
  
  
  EnergyFermiSurface = ...
      ones(number_of_spin, Hdim, NKgrid(1), NKgrid(2), NKgrid(3)) * 1000;
  
  for ispin = 1 : number_of_spin,
    fprintf(1,'\nispin= %d   iKX= ',ispin);
    for iKX = 1 : NKgrid(1),
      fprintf(1,'\n %d    ',iKX);
      fprintf(1,'iKY= ');
      for iKY = 1 : NKgrid(2),
        fprintf(1,'%d  ',iKY);
        for iKZ = 1 : NKgrid(3),
          
          Kvector = KX(:,iKX) + KY(:,iKY) + KZ(:,iKZ);
          
          if (Lrcutoff == 0),
            matPhase = exp( i *( RCellPos(:,:) * H ) * Kvector);
          end;
          
          HH = zeros(Hdim, Hdim);
          SS = zeros(Hdim, Hdim);
          
          OrbX  = 0;
          OrbXX = 0;
          for iatom = 1 : atomMax,
            itype = atomtype(iatom);
            for iorb = 1 : orbMax(itype);
              OrbXX = OrbX + iorb;
              
              OrbY = 0;
              OrbYY = 0;
              for jatom = 1 : atomMax,
                jtype = atomtype(jatom);
                
                if (Lrcutoff ~= 0),
                  matPhaseCutoff = ...
                      exp( i *( squeeze(RCellPosTemp(iatom, jatom, 1:RCellIDMax(iatom,jatom) , 1:3)) * H ) * Kvector);
                  matIDCutoff   = ...
                      squeeze(RCellIDTemp(iatom, jatom, 1:RCellIDMax(iatom,jatom)));
                end;
                
                for jorb = 1 : orbMax(jtype);
                  OrbYY = OrbY + jorb;
                  
                  if (Lrcutoff == 0),
                    HH(OrbXX, OrbYY) = ...
                        HH(OrbXX, OrbYY) + ...
                        sum( squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, :)) .* ...
                             matPhase );
                    
                    SS(OrbXX, OrbYY) = ...
                        SS(OrbXX, OrbYY) + ...
                        sum( squeeze(TBSOverlap(ispin, OrbXX, OrbYY, :)) .*  ...
                             matPhase );
                  else                
                    HH(OrbXX, OrbYY) = ...
                        HH(OrbXX, OrbYY) + ...
                        sum( ...
                            squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, matIDCutoff )) .* ...
                            matPhaseCutoff);
                    SS(OrbXX, OrbYY) = ...
                        SS(OrbXX, OrbYY) + ...
                        sum( ...
                            squeeze(TBSOverlap(ispin, OrbXX, OrbYY, matIDCutoff)) .*  ...
                            matPhaseCutoff );                  
                  end;
                  
                end;
                OrbY = OrbY + orbMax(jtype);
              end;
              
            end;
            OrbX = OrbX + orbMax(itype);
          end;
          
          dd = GenEig(HH, SS, GenEigThreshold);
          
          EnergyFermiSurface(ispin, 1:length(dd), iKX, iKY, iKZ) = ...
              sort(real(dd))';
          
        end;
      end;
    end;
    
  end;
  
  origin = [0 0 0];
  
  if number_of_spin == 1,
    ispin = 1;
    FermiData = squeeze( EnergyFermiSurface(ispin, :, :, :, :) );
    [ierror,str_error] = ...
        QuickOutputBXSF('FermiSurface', fname, ...
                        FermiData, 3, NKgrid, ...
                        Hdim, origin, G, Efermi);
    fprintf(1,['\n[QuickFermiSurface] BXSF file generated: \n' fname '\n'] );
    
  else
    for ispin = 1 : number_of_spin,
      if (ispin == 1),
        fermifilename = [fname '.spin.up.bxsf'];
        ftag = ['FermiSurface.Spin.Up'];     
      else
        fermifilename = [fname '.spin.down.bxsf'];
        ftag = ['FermiSurface.Spin.Down'];
      end;
      
      FermiData = squeeze( EnergyFermiSurface(ispin, :, :, :, :));
      [ierror,str_error] = ...
          QuickOutputBXSF(ftag, fermifilename, ...
                          FermiData, 3, NKgrid, ...
                          Hdim, origin, G, Efermi);
      fprintf(1,['\n[QuickFermiSurface] BXSF file generated: ' fname '\n'] );
      
    end;
  end;
  
  
  QMB.EnergyFermiSurface  = QMB;

  fprintf(1,'\n\n');
  
  return;
  
  
  
