% QuickPlotTBBandStructure.m %

function [iflag_PlotBandStructure] = QuickPlotTBBandStructure(Input, QMB)
  
  iflag_PlotBandStructure = 0;
  
  fprintf (1, 'QuickPlotTBBandStructure: Plot out tightbinding band structure\n');
  
  Eshiftplot  = Input.CombineUnoccupiedBV.EFermiShift;
  EnergyRange = Input.PlotBandStructure.EnergyRange;
  bsfilename  = Input.PlotBandStructure.Filename;
  
  TBBandStructure = QMB.TBBandStructure;
  Efermi = QMB.Efermi;
  Hdim   = size(QMB.TBHamOverlap, 2);
  NKgrid = size(TBBandStructure.Energy, 4);
  Kpath  = TBBandStructure.Kpath;
  number_of_spin = QMB.nSpins;
  BS = QMB.TBBandStructure.BandData;
  
  lwidth = 2;
  

  set(0,'DefaultAxesXGrid','off');
  set(0,'DefaultAxesYGrid','off');
  set(0,'DefaultAxesFontSize',14);
  set(0,'DefaultTextFontSize',14);
  set(0,'DefaultAxesFontSize',18);
  set(0,'DefaultTextFontSize',18);
  set(0,'DefaultAxesLineWidth',1);
  set(0,'DefaultLineLineWidth',2);
  set(0,'DefaultAxesFontName','times');
  set(0,'DefaultTextFontName','times');
  
for ispin = 1: number_of_spin,
  
  hfig =figure('visible', 'off');
  clf;
  hold on;
  
  for iKpath = 1 : (size(Kpath,2)-1),
    if ( TBBandStructure.KpathLabel(iKpath) == 'I' ) || ( TBBandStructure.KpathLabel(iKpath+1) == 'I' ) ,
      plot(squeeze(BS(ispin, 1, (NKgrid*(iKpath-1)+1):NKgrid*iKpath )), ...
           squeeze(BS(ispin, 2:Hdim+1,(NKgrid*(iKpath-1)+1):NKgrid*iKpath )-Efermi),...
           'm--','linewidth',lwidth);
    else
      plot(squeeze(BS(ispin, 1, (NKgrid*(iKpath-1)+1):NKgrid*iKpath )), ...
           squeeze(BS(ispin, 2:Hdim+1,(NKgrid*(iKpath-1)+1):NKgrid*iKpath )-Efermi), ...
           'linewidth',lwidth,'color',[0 0 0]);
    end;
  end;
  
  axis( [BS(ispin, 1,1)-0.5 BS(ispin, 1,NKgrid*(size(Kpath,2)-1)) EnergyRange(1) EnergyRange(2) ]);
  hold on;
  
%   plot(squeeze(BS(ispin, 1,:)),0*ones(size(BS,3),1) , 'm-', 'linewidth',lwidth); %
  plot(squeeze(BS(ispin, 1,:)),0*ones(size(BS,3),1) , 'k--', 'linewidth',lwidth);
  if (abs(Eshiftplot) > 1e-6),
    plot(squeeze(BS(ispin, 1,:)),Eshiftplot*ones(size(BS,3),1) , 'r-.', 'linewidth',lwidth);
  end;
  
  xxtick = zeros(size(Kpath,2)-1,1);
  for iKpath = 0 : (size(Kpath,2)-2),
    xxtick(iKpath+1,1) = BS(ispin, 1,NKgrid*iKpath+1);
    line( [ BS(ispin, 1,NKgrid*iKpath+1) BS(ispin, 1,NKgrid*iKpath+1) ], ...
          [EnergyRange(1) EnergyRange(2)], ...
          'linewidth',2,'Color',[0 0 0]);
  end;
  
  iKpath = size(Kpath,2)-1;
  line( [ BS(ispin, 1,NKgrid*iKpath) BS(ispin, 1,NKgrid*iKpath) ], ...
        [EnergyRange(1) EnergyRange(2)], ...
        'linewidth',2,'Color',[0 0 0]);
  xxtick(iKpath+1,1) = BS(ispin, 1,NKgrid*iKpath);
  
  set(gca,'XTick', xxtick);
  set(gca,'XTickLabel',TBBandStructure.KpathLabel(:))
  
  ylabel('Energy (eV)');
%   grid on; %
  grid off;
  box on;
  drawnow;
  
  if ( number_of_spin == 2),
    if (ispin == 1),
      print('-depsc', '-r300', [bsfilename '.spin.up.eps'] );
    else
      print('-depsc', '-r300', [bsfilename '.spin.down.eps'] );
    end;
  else
    print('-depsc', '-r300', [bsfilename] );
    % printf(1,['\n[QuickPlotTBBandStructure] EPS file generated: '  '\n']; %
  end;
  
  fprintf (1, '\n');
  close(hfig);
  
end;

iflag_PlotBandStructure = 1;

return;

