% QuickTBOverlap.m %

function [ QMB ] = QuickTBOverlap(tbdatafile, Atom, PsiCoeff, Psp, PW, BlochBasis, QMB)
  
  fprintf (1, '[QuickTBOverlap]  Calculate tightbinding hopping and overlaping matrix\n');
  
  number_of_bands = QMB.number_of_bands;
  number_of_spin  = PW.nSpins;
  Enk        = PW.EigenValues;
  Rdim       = PW.KpointSetup;
  IBZKWeight = PW.IBZKWeight;
  IBZK       = PW.IBZK;
  atomMax    = Atom.AtomMax;
  atomtype   = Atom.Atom2Type;
  H          = Atom.H;
  
  
  Hdim = 0;
  for iatom = 1:atomMax,
    itype = atomtype(iatom);
    Hdim  = Hdim + Psp{itype}.orbMax;
  end;
  
  
  Rdim = floor(abs(Rdim-0.5)/2);
  
  clear RCellID RCellPos;
  id = 0;
  for ixx = -Rdim(1) : Rdim(1),
    ix = ixx + Rdim(1) + 1;
    for iyy = -Rdim(2) : Rdim(2),
      iy = iyy + Rdim(2) + 1;
      for izz = -Rdim(3) : Rdim(3),
        iz = izz + Rdim(3) + 1;
        
        id = id + 1;
        RCellID(id, :)  = [ix iy iz];
        RCellPos(id, :) = [ixx iyy izz];
        
      end;
    end;
  end;
  TotalCell = size(RCellID,1);
  
  % store data: % 
  % <quambo_t0 | H | quambo_tpR> %
  % <quambo_t0 | S | quambo_tpR> %
  TBHamOverlap = zeros( number_of_spin, Hdim, Hdim, TotalCell);
  TBSOverlap   = zeros( number_of_spin, Hdim, Hdim, TotalCell);
  
  tmppos = exp( - i *( RCellPos(:,:) * H )* IBZK(:,:)' ).';
  
  OrbX  = 0;
  OrbXX = 0;
  
  for iatom = 1 : atomMax,
    itype = atomtype(iatom);
    fprintf(1,'\niatom = %d,  Orb:  ',iatom);
    
    for iorb = 1 : Psp{itype}.orbMax;
      OrbXX = OrbX + iorb;
      
      fprintf(1,'%d ', iorb);
      
      OrbY = 0;
      OrbYY = 0;
      for jatom = 1 : atomMax,
        jtype = atomtype(jatom);
        for jorb = 1 : Psp{jtype}.orbMax;
          OrbYY = OrbY + jorb;
          
          for ispin = 1 : number_of_spin,
            for iband = 1 : number_of_bands,
              
              tmpvector = ...
                  IBZKWeight(:) .* ...
                  conj(PsiCoeff(:, ispin, iband, iatom, iorb)) .* ...
                  PsiCoeff(:, ispin, iband, jatom, jorb);
              
              TBHamOverlap(ispin, OrbXX, OrbYY, :) = ...
                  TBHamOverlap(ispin, OrbXX, OrbYY, :) + ...
                  reshape(real( (Enk(:, ispin, iband) .* tmpvector).' * ...
                                tmppos), ...
                          1,1,1, TotalCell);
              
              TBSOverlap(ispin, OrbXX, OrbYY, :) = ...
                  TBSOverlap(ispin, OrbXX, OrbYY, :) + ...
                  reshape(real( tmpvector.' * ...
                                tmppos), ...
                          1,1,1, TotalCell);
            end;
          end;
          
        end;
        OrbY = OrbY + Psp{jtype}.orbMax;
      end;
      
    end;
    OrbX = OrbX + Psp{itype}.orbMax;
    
  end;
  
  QMB.Rdim = Rdim;
  QMB.TBHamOverlap = TBHamOverlap;
  QMB.TBSOverlap = TBSOverlap;
  QMB.Enk_PW     = Enk;
  QMB.nSpins     = PW.nSpins;
  QMB.nBands     = PW.nBands;
  QMB.Efermi     = PW.Efermi;
  QMB.atomMax    = Atom.AtomMax;
  QMB.atomtype   = Atom.Atom2Type;
  QMB.atomsymbol = Atom.AtomSymbol;
  QMB.G          = Atom.G;
  QMB.H          = Atom.H;
  QMB.atom_X     = Atom.X;
  QMB.IBZK       = PW.IBZK;
  QMB.IBZKWeight = PW.IBZKWeight;
  QMB.IBZKMax    = PW.IBZKMax;
  QMB.oldIBZK    = PW.oldIBZK;
  QMB.oldIBZKWeight = PW.oldIBZKWeight;
  QMB.oldIBZKMax    = PW.oldIBZKMax;
  QMB.BZKMax        = PW.BZKMax;
  
  
  QMB.orbMax = zeros(Psp{1}.TypeMax,1);
  for itype = 1: Psp{1}.TypeMax,
    QMB.orbMax(itype) = Psp{itype}.orbMax;
  end;
  
  
  save(tbdatafile,'QMB',  'PsiCoeff', 'BlochBasis', 'Atom', 'PW', 'Psp');
  
  fprintf (1, '\n\n');
  
