% GIndexGenerator.m %
% Generate GIndex for different IBZKpoints eithor coarse grid or dense grid %
% WaveFunctionFFTindex(kx,ky,kz) --> GIndex %
% ibz:    IBZKpoint %
% itype:  0=coarse 1=dense%
% idebug: debug flag 0=No 1=Yes %
% [GIndex ,ingx, ingy, ingz] = GIndexGenerator(FFTIndex, itype, ibz); %

function [GIndex,ingx,ingy,ingz] = ...
      GIndexGenerator(WaveFunctionFFTindex,itype,ibz,idebug)

  
  lenx = max(WaveFunctionFFTindex(ibz,1,:));
  leny = max(WaveFunctionFFTindex(ibz,2,:));
  lenz = max(WaveFunctionFFTindex(ibz,3,:));
  
%   [lenx leny lenz] %
  
  %coarse
  if (itype == 0)
    NGX = round(lenx/2);
    NGY = round(leny/2);
    NGZ = round(lenz/2);
    
    indexcoarse = zeros(length(WaveFunctionFFTindex(ibz,1,:)),1);
    inx = squeeze(WaveFunctionFFTindex(ibz,1,:)) - ...
          NGX*(squeeze(WaveFunctionFFTindex(ibz,1,:))>NGX);
    iny = squeeze(WaveFunctionFFTindex(ibz,2,:)) - ...
          NGY*(squeeze(WaveFunctionFFTindex(ibz,2,:))>NGY);
    inz = squeeze(WaveFunctionFFTindex(ibz,3,:)) - ...
          NGZ*(squeeze(WaveFunctionFFTindex(ibz,3,:))>NGZ);
    % for crystal %
    %     indexcoarse(:,1) = inx  + (iny-1)*NGX + (inz-1)*NGX*NGY; %
    % for molecule     %
    indexcoarse(:) = inx  + (iny-1)*NGX + (inz-1)*NGX*NGY;
    
    if (idebug == 1)
      global WaveFunction;
      fprintf(1,'[Debug] GListCoarse / WaveFunction\n');
      wavtest = 1293;
      ispin = 1;
      iband = 1;
      GListCoarse = zeros(NGX*NGY*NGZ,1);
      wavetest2 = WaveFunctionFFTindex(ibz,:,wavtest);
      GListCoarseTest(indexcoarse(:,1)) = ...
          WaveFunction(ibz,ispin,iband,:,1) + ...
          j*WaveFunction(ibz,ispin,iband,:,2);
      GCubeTest = reshape(GListCoarseTest,NGX,NGY,NGZ);
      GCubeTest(wavetest2(1)-NGX*(wavetest2(1)>NGX), ...
                wavetest2(2)-NGY*(wavetest2(2)>NGY),...
                wavetest2(3)-NGZ*(wavetest2(3)>NGZ))
      wav = WaveFunction(ibz,ispin,iband,wavtest,1) + ...
            WaveFunction(ibz,ispin,iband,wavtest,2)*j
    end;
    
    GIndex = indexcoarse;
    ingx = NGX;
    ingy = NGY;
    ingz = NGZ;
    
    return;
    
    %dense
  elseif (itype == 1)
    indexdense = zeros(length(squeeze(WaveFunctionFFTindex(ibz,1,:))),1);
    indexdense(:,1) = ( squeeze(WaveFunctionFFTindex(ibz,1,:)) + ...
                        (squeeze(WaveFunctionFFTindex(ibz,2,:))-1)*lenx + ...
                        (squeeze(WaveFunctionFFTindex(ibz,3,:))-1)*lenx*leny);
    
    if (idebug == 1)
      global WaveFunction;
      fprintf(1,'[Debug] GListDense / WaveFunction\n');
      wavtest = 1293;
      GListDense  = zeros(lenx*leny*lenz,1);
      ispin = 1;
      iband = 1;      
      GListDenseTest(indexdense(:,1)) = WaveFunction(ibz,ispin,iband,:,1) + j*WaveFunction(ibz,ispin,iband,:,2);
      wavetest2 = WaveFunctionFFTindex(ibz,:,wavtest);
      GCubeTest = reshape(GListDenseTest,lenx,leny,lenz);
      GCubeTest(wavetest2(1),wavetest2(2),wavetest2(3))
      wav = WaveFunction(ibz,ispin,iband,wavtest,1)+ WaveFunction(ibz,ispin,iband,wavtest,2)*j
    end;
    
    GIndex = indexdense;
    ingx = lenx;
    ingy = leny;
    ingz = lenz;
    
    return;
    
  else
    fprintf(1,'[GIndexGenerator] itype(coarse/dense) is not defined \n]');
    pause;
    return;
  end;

  
  % WaveFunctionFFTindex(number_IBZ_kpoints,3,number_plane_waves) %
  % nc{'WaveFunction'} = ncfloat('number_IBZ_kpoints', 'number_of_spin', ... %
  %                              'number_of_bands', 'number_plane_waves', 'real_complex');  %
