% GenXcrysdenScript.m %
% Purpose: Generate xcrysden tcl script %
%          use xsf or [ss] = xsf(script_fname) to visualize  %

function [script_fname] = GenXcrysdenScript(xsf_fname, script_fname, Param);
  
  fid_xsf = fopen(script_fname,'wt');
  if (fid_xsf<0)
    str_error = ['GenXcrysdenScript: Cannot creat file ' system];
    return;
  end;
  
  fprintf(fid_xsf,'scripting::open --xsf %s\n', xsf_fname); 
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'set myParam(BACKGROUND)      {1.00 1.00 1.00}\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'set myParam(BALLF)           0.4\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'set myParam(RODF)            0.6\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'scripting::load_myParam\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'scripting::lighting On\n'); 
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'scripting::displayMode3D BallStick\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'scripting::scalarField3D::load\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'IsoControl_Hide\n');
  fprintf(fid_xsf,'\n');
  tmpstr = ['set isolevel [expr 0.9*$isosurf(minvalue)+0.1*$isosurf(maxvalue)] \n'];
  fprintf(fid_xsf,...
          tmpstr);
  fprintf(fid_xsf,'\n');
  
  if norm(Param.Isoplane)>1e-2,
    slide = Param.Grid * Param.Spline;
    cslide = round(slide/2 + 1e-5) + 1;
    for ip = 1 : 3,
      if Param.Isoplane(ip) >= 0.999999,
        fprintf(fid_xsf,'scripting::scalarField3D::configure \\\n');
        fprintf(fid_xsf,'-isolevel $isolevel \\\n');
        fprintf(fid_xsf,'-isosurface           1 \\\n');
        fprintf(fid_xsf,['-interpolation_degree ' num2str(Param.Spline) ' \\\n']);
        fprintf(fid_xsf,'-plusminus            0 \\\n');
        fprintf(fid_xsf,['-basalplane           ' num2str(ip) ' \\\n']);
        fprintf(fid_xsf,'-colorbasis           RAINBOW \\\n');
        fprintf(fid_xsf,'-scalefunction        LINEAR \\\n');
        fprintf(fid_xsf,'-expand2D             specify \\\n');
        fprintf(fid_xsf,'-expand2D_X           1 \\\n');
        fprintf(fid_xsf,'-expand2D_Y           1 \\\n');
        fprintf(fid_xsf,'-expand2D_Z           1 \\\n');
        fprintf(fid_xsf,'-colorplane           1 \\\n');
        fprintf(fid_xsf,'-isoline              1 \\\n');
        fprintf(fid_xsf,'-colorplane_lighting  1 \\\n');
        fprintf(fid_xsf,'-cpl_transparency     0 \\\n');
        fprintf(fid_xsf,'-cpl_thermometer      0 \\\n');
        fprintf(fid_xsf,'-2Dnisoline           45 \\\n');
        fprintf(fid_xsf,'-anim_step            1 \\\n');
        fprintf(fid_xsf,['-current_slide        ' num2str(cslide(ip)) ' \\\n']);
        fprintf(fid_xsf,'-isoline_color        {property color} \\\n');
        fprintf(fid_xsf,'-isoline_width        1\n');
        fprintf(fid_xsf,'\n');
      end;
    end;
    
  else
      fprintf(fid_xsf,'scripting::scalarField3D::configure \\\n');
      fprintf(fid_xsf,'-isolevel $isolevel \\\n');
      fprintf(fid_xsf,'-isosurface           1 \\\n');
      fprintf(fid_xsf,'-interpolation_degree 1 \\\n');
      fprintf(fid_xsf,'-plusminus            0 \\\n');
      fprintf(fid_xsf,'-expand               whole\n');
      fprintf(fid_xsf,'\n');
  end;
  
  fprintf(fid_xsf,'scripting::buildCrystal 1 1 1\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'scripting::zoom -0.08 8\n');
  fprintf(fid_xsf,'\n');
  fprintf(fid_xsf,'scripting::scalarField3D::render\n');
  fprintf(fid_xsf,'\n');
  
  fclose(fid_xsf);
  
  fprintf(1,...
          '[GenXcrysdenScript] Succeed in gernerating tcl script: %s', ...
          script_fname);
  fprintf(1,'\n\n');
  
