% QuickOutputBXSF.m %
% Generate Fermi surface data for BXSF structure %

function [ierror,str_error] = ...
      QuickOutputBXSF(xsfblock,xsfname,data,idim, datasize ,nband, origin, G, Efermi)
  
  ingx = datasize(1);
  ingy = datasize(2);
  ingz = datasize(3);
  
  fid_xsf = fopen([xsfname],'wt');
  if (fid_xsf<0)
    str_error = ['QuickOutputBXSF: Cannot creat file ' xsfname '.'];
    return;
  end;
  
  fprintf(fid_xsf,'BEGIN_INFO\n');
  fprintf(fid_xsf,'  # Launch as: xcrysden --bxsf ex.bxsf\n');
  fprintf(fid_xsf,'  Fermi Energy:  %f\n',Efermi);
  fprintf(fid_xsf,'END_INFO\n');
  
  if (idim == 3)
    fprintf(fid_xsf,'BEGIN_BLOCK_BANDGRID_3D\n'); 
    fprintf(fid_xsf,'FermiSurface_%s\n',xsfblock);
    fprintf(fid_xsf,'BANDGRID_3D_BANDS\n');
    fprintf(fid_xsf,'%d\n',nband);
    fprintf(fid_xsf,'%d %d %d\n',ingx,ingy,ingz);
    fprintf(fid_xsf,'%f %f %f\n',origin(1),origin(2),origin(3));
    fprintf(fid_xsf,'%f %f %f\n',G(1,1),G(2,1),G(3,1));
    fprintf(fid_xsf,'%f %f %f\n',G(1,2),G(2,2),G(3,2));
    fprintf(fid_xsf,'%f %f %f\n',G(1,3),G(2,3),G(3,3));
    
    id=0;
    for iband = 1 : nband,
      fprintf(fid_xsf,'BAND:  %d\n',iband);
      
      for ix = 1:ingx,
        for iy = 1:ingy,
          for iz = 1:ingz,
            fprintf(fid_xsf,'%f ',data(iband,ix,iy,iz));
            id = id + 1;
            if (id == 8) 
              id =0;
              fprintf(fid_xsf,'\n');
            end;
          end;
        end;
      end;
      
      fprintf(fid_xsf,'\n');
    end;
    
    fprintf(fid_xsf,'END_BANDGRID_3D\n');
    fprintf(fid_xsf,'END_BLOCK_BANDGRID_3D\n');
    fclose(fid_xsf);
    % fprintf(1,'[XOutput] Succeed Generating file: %s \n',xsfname);
    ierror = 0;
    str_error = ['QuickOutputBXSF Succeed Generating file: ' xsfname];
    return;
  
  else 
    ierror = 1;
    str_error = 'Error: QuickOutputBXSF Dimension should be 3. \n';
    return;
  end;
  
